import Flutter
import UIKit
import PixelPaySDK

public class SwiftPixelpaySdkPlugin: NSObject, FlutterPlugin {
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "pixelpay_sdk", binaryMessenger: registrar.messenger())
        let instance = SwiftPixelpaySdkPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
            /**
            * Transaction service
            */
            case "transactionDoSale": self.transactionDoSale(call: call, result: result); break;
            case "transactionDoAuth": self.transactionDoAuth(call: call, result: result); break;
            case "transactionDoCapture": self.transactionDoCapture(call: call, result: result); break;
            case "transactionDoVoid": self.transactionDoVoid(call: call, result: result); break;
            case "transactionGetStatus": self.transactionGetStatus(call: call, result: result); break;

            /**
            * Tokenization service
            */
            case "tokenizationVaultCard": self.tokenizationVaultCard(call: call, result: result); break;
            case "tokenizationUpdateCard": self.tokenizationUpdateCard(call: call, result: result); break;
            case "tokenizationShowCard": self.tokenizationShowCard(call: call, result: result); break;
            case "tokenizationShowCards": self.tokenizationShowCards(call: call, result: result); break;
            case "tokenizationDeleteCard": self.tokenizationDeleteCard(call: call, result: result); break;

            default: result(FlutterMethodNotImplemented); break;
        }
    }
    
    internal func fromJson<T>(type: T.Type, json: String) throws -> T where T : Decodable {
        let decoder = JSONDecoder();
        
        return try decoder.decode(type, from: Data(json.utf8))
    }
    
    internal func getTransactionService(settings: String) throws -> Transaction {
        return Transaction(settings: try self.fromJson(type: Settings.self, json: settings));
    }

    internal func getTokenizationService(settings: String) throws -> Tokenization {
        return Tokenization(settings: try self.fromJson(type: Settings.self, json: settings));
    }
    
    internal func parseSuccess(result: @escaping FlutterResult, response: Response) {
        result([
            "status": String(response.getStatus() ?? 500),
            "response": response.toJson()
        ])
    }

    internal func parseError(result: @escaping FlutterResult, error: Error, code: String) {
        result(FlutterError(code: code, message: error.localizedDescription, details: nil));
    }
    
    internal func transactionDoSale(call: FlutterMethodCall, result: @escaping FlutterResult) {
        do {
            if let args = call.arguments as? Dictionary<String, Any>,
               let settings = args["settings"] as? String,
               let request = args["request"] as? String {
                
                try self.getTransactionService(settings: settings)
                    .doSale(transaction: self.fromJson(type: SaleTransaction.self, json: request)) { response in
                        self.parseSuccess(result: result, response: response)
                    }
            } else {
                result(FlutterError.init(code: "Invalid request argument to MethodChannel", message: nil, details: nil))
            }
        } catch {
            self.parseError(result: result, error: error, code: "transaction-error")
        }
    }
    
    internal func transactionDoAuth(call: FlutterMethodCall, result: @escaping FlutterResult) {
        do {
            if let args = call.arguments as? Dictionary<String, Any>,
               let settings = args["settings"] as? String,
               let request = args["request"] as? String {
                
                try self.getTransactionService(settings: settings)
                    .doAuth(transaction: self.fromJson(type: AuthTransaction.self, json: request)) { response in
                        self.parseSuccess(result: result, response: response)
                    }
            } else {
                result(FlutterError.init(code: "Invalid request argument to MethodChannel", message: nil, details: nil))
            }
        } catch {
            self.parseError(result: result, error: error, code: "transaction-error")
        }
    }
    
    internal func transactionDoCapture(call: FlutterMethodCall, result: @escaping FlutterResult) {
        do {
            if let args = call.arguments as? Dictionary<String, Any>,
               let settings = args["settings"] as? String,
               let request = args["request"] as? String {
                
                try self.getTransactionService(settings: settings)
                    .doCapture(transaction: self.fromJson(type: CaptureTransaction.self, json: request)) { response in
                        self.parseSuccess(result: result, response: response)
                    }
            } else {
                result(FlutterError.init(code: "Invalid request argument to MethodChannel", message: nil, details: nil))
            }
        } catch {
            self.parseError(result: result, error: error, code: "transaction-error")
        }
    }
    
    internal func transactionDoVoid(call: FlutterMethodCall, result: @escaping FlutterResult) {
        do {
            if let args = call.arguments as? Dictionary<String, Any>,
               let settings = args["settings"] as? String,
               let request = args["request"] as? String {
                
                try self.getTransactionService(settings: settings)
                    .doVoid(transaction: self.fromJson(type: VoidTransaction.self, json: request)) { response in
                        self.parseSuccess(result: result, response: response)
                    }
            } else {
                result(FlutterError.init(code: "Invalid request argument to MethodChannel", message: nil, details: nil))
            }
        } catch {
            self.parseError(result: result, error: error, code: "transaction-error")
        }
    }
    
    internal func transactionGetStatus(call: FlutterMethodCall, result: @escaping FlutterResult) {
        do {
            if let args = call.arguments as? Dictionary<String, Any>,
               let settings = args["settings"] as? String,
               let request = args["request"] as? String {
                
                try self.getTransactionService(settings: settings)
                    .getStatus(transaction: self.fromJson(type: StatusTransaction.self, json: request)) { response in
                        self.parseSuccess(result: result, response: response)
                    }
            } else {
                result(FlutterError.init(code: "Invalid request argument to MethodChannel", message: nil, details: nil))
            }
        } catch {
            self.parseError(result: result, error: error, code: "transaction-error")
        }
    }
    
    internal func tokenizationVaultCard(call: FlutterMethodCall, result: @escaping FlutterResult) {
        do {
            if let args = call.arguments as? Dictionary<String, Any>,
               let settings = args["settings"] as? String,
               let request = args["request"] as? String {
                
                try self.getTokenizationService(settings: settings)
                    .vaultCard(card: self.fromJson(type: CardTokenization.self, json: request)) { response in
                        self.parseSuccess(result: result, response: response)
                    }
            } else {
                result(FlutterError.init(code: "Invalid request argument to MethodChannel", message: nil, details: nil))
            }
        } catch {
            self.parseError(result: result, error: error, code: "tokenization-error")
        }
    }
    
    internal func tokenizationUpdateCard(call: FlutterMethodCall, result: @escaping FlutterResult) {
        do {
            if let args = call.arguments as? Dictionary<String, Any>,
               let settings = args["settings"] as? String,
               let token = args["token"] as? String,
               let request = args["request"] as? String {
                
                try self.getTokenizationService(settings: settings)
                    .updateCard(token: token, card: self.fromJson(type: CardTokenization.self, json: request)) { response in
                        self.parseSuccess(result: result, response: response)
                    }
            } else {
                result(FlutterError.init(code: "Invalid request argument to MethodChannel", message: nil, details: nil))
            }
        } catch {
            self.parseError(result: result, error: error, code: "tokenization-error")
        }
    }
    
    internal func tokenizationShowCard(call: FlutterMethodCall, result: @escaping FlutterResult) {
        do {
            if let args = call.arguments as? Dictionary<String, Any>,
               let settings = args["settings"] as? String,
               let token = args["token"] as? String {
                
                try self.getTokenizationService(settings: settings)
                    .showCard(token: token) { response in
                        self.parseSuccess(result: result, response: response)
                    }
            } else {
                result(FlutterError.init(code: "Invalid request argument to MethodChannel", message: nil, details: nil))
            }
        } catch {
            self.parseError(result: result, error: error, code: "tokenization-error")
        }
    }
    
    internal func tokenizationShowCards(call: FlutterMethodCall, result: @escaping FlutterResult) {
        do {
            if let args = call.arguments as? Dictionary<String, Any>,
               let settings = args["settings"] as? String,
               let tokens = args["tokens"] as? Array<String> {
                
                try self.getTokenizationService(settings: settings)
                    .showCards(tokens: tokens) { response in
                        self.parseSuccess(result: result, response: response)
                    }
            } else {
                result(FlutterError.init(code: "Invalid request argument to MethodChannel", message: nil, details: nil))
            }
        } catch {
            self.parseError(result: result, error: error, code: "tokenization-error")
        }
    }
    
    internal func tokenizationDeleteCard(call: FlutterMethodCall, result: @escaping FlutterResult) {
        do {
            if let args = call.arguments as? Dictionary<String, Any>,
               let settings = args["settings"] as? String,
               let token = args["token"] as? String {
                
                try self.getTokenizationService(settings: settings)
                    .deleteCard(token: token) { response in
                        self.parseSuccess(result: result, response: response)
                    }
            } else {
                result(FlutterError.init(code: "Invalid request argument to MethodChannel", message: nil, details: nil))
            }
        } catch {
            self.parseError(result: result, error: error, code: "tokenization-error")
        }
    }
}
